<?php

declare(strict_types=1);

namespace Drupal\track_usage;

use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\track_usage\Entity\TrackConfigInterface;
use Drupal\track_usage\Model\EntityRole;
use Drupal\track_usage\Trait\EntityUtilityTrait;
use Drupal\track_usage\Trait\IdColumnTrait;

/**
 * Service used to query recorder usages.
 */
class Reader implements ReaderInterface {

  use EntityUtilityTrait;
  use IdColumnTrait;

  public function __construct(
    protected readonly Connection $db,
    protected readonly EntityTypeManagerInterface $entityTypeManager,
    protected readonly EntityFieldManagerInterface $entityFieldManager,
  ) {}

  /**
   * {@inheritdoc}
   */
  public function getTargetsForSourceForTarget(EntityInterface $source, TrackConfigInterface $config): array {
    if (!$config->isSource($source)) {
      throw new \InvalidArgumentException("The {$source->getEntityType()->getLabel()} entity with ID {$source->id()} entity is not defined as source entity in '{$config->id()}' configuration");
    }

    $query = $this->db->select(RecorderInterface::TABLE)
      ->fields(RecorderInterface::TABLE, ['target_type', 'target_id_int', 'target_id_string'])
      ->condition('source_entity_type', $source->getEntityTypeId())
      ->condition($this->getIdColumnName($source, EntityRole::Source), $source->id())
      ->condition('source_language', $source->language()->getId());

    if ($this->isRevisionable($source)) {
      $query->condition('source_revision', $this->getRevisionId($source));
    }

    $targetEntities = [];
    foreach ($query->execute() as $row) {
      $column = $this->getIdColumnName($row['target_type'], EntityRole::Target);
      $id = $this->getIdColumnValue($row['target_type'], $row[$column]);
      $targetEntities[$row['target_type']][$id] = $id;
    }

    return $targetEntities;
  }

}
