<?php

declare(strict_types=1);

namespace Drupal\track_usage;

use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\track_usage\Entity\TrackConfigInterface;

/**
 * Reads recorder usages.
 */
interface ReaderInterface {

  /**
   * Returns all recorder usages for a target entity.
   *
   * @param \Drupal\Core\Entity\FieldableEntityInterface $source
   *   The source entity.
   * @param \Drupal\track_usage\Entity\TrackConfigInterface $config
   *   The track usage configuration to use.
   *
   * @return array<array-key, list<int|string>
   *   Associative array with entity type IDs as keys and entity IDs as values.
   *
   * @throws \InvalidArgumentException
   *   When the passed entity is not a source entity.
   */
  public function getTargetsForSourceForTarget(FieldableEntityInterface $source, TrackConfigInterface $config): array;

}
