<?php

declare(strict_types=1);

namespace Drupal\track_usage\Trait;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\RevisionableInterface;

/**
 * Reusable utility code.
 */
trait EntityUtilityTrait {

  /**
   * Returns the entity revision.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity revision or 0 if isn't supported.
   */
  protected function getRevisionId(EntityInterface $entity): int {
    return $entity instanceof RevisionableInterface ? (int) $entity->getRevisionId() : 0;
  }

  /**
   * Builds an "entity key".
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   Entity type, entity ID and revision separated by colon.
   */
  protected function getKey(EntityInterface $entity): string {
    return "{$entity->getEntityTypeId()}:{$entity->id()}:{$this->getRevisionId($entity)}";
  }

  /**
   * Checks whether an entity type is revisionable.
   *
   * @param \Drupal\Core\Entity\EntityInterface|string $entityOrEntityTypeId
   *   The entity or the entity type ID.
   *
   * @return bool
   *   Whether an entity type is revisionable.
   */
  protected static function isRevisionable(EntityInterface|string $entityOrEntityTypeId): bool {
    $entityTypeId = $entityOrEntityTypeId instanceof EntityInterface ? $entityOrEntityTypeId->getEntityTypeId() : $entityOrEntityTypeId;
    $entityType = \Drupal::entityTypeManager()->getDefinition($entityTypeId);
    return $entityType->entityClassImplements(RevisionableInterface::class) && $entityType->hasKey('revision');
  }

}
